#!/bin/bash
#
#
# COPYRIGHT 2003-2013, EMULEX CORPORATION                               
# 3333 Susan St., Costa Mesa, CA 92626                                  
#                                                                         
# All rights reserved.  This computer program and related documentation 
# is protected by copyright  and distributed under licenses restricting 
# its use,  copying,  distribution  and decompilation.    This computer 
# program  and its  documentation  are CONFIDENTIAL  and a TRADE SECRET 
# of EMULEX CORPORATION.   The receipt or  possession of  this  program 
# or its documentation does not  convey rights to reproduce or disclose 
# its  contents,  or to  manufacture, use, or sell anything that it may 
# describe, in whole or in part,  without the specific  written consent 
# of  EMULEX CORPORATION.   Any reproduction  of  this program  without 
# the express  written  consent  of EMULEX  CORPORATION  is a violation 
# of the  copyright laws  and may  subject you to  criminal prosecution.


# Get environment data
OS=`uname -s`
Arch=`uname -m`
if [ "$Arch" = "i686" ] || [ "$Arch" = "i585" ];then
    Arch="i386"
fi
DIR=$(cd ${0%/*} ; pwd )

SUCCESS=0
FAILURE=1
ocv_install_error=0
LOGFILE_NAME="utils-install.log"
INSTALL_DIR="usr/sbin/ocmanager"
LOG_FILE="/${INSTALL_DIR}/${LOGFILE_NAME}"
PRINT_TO_CONSOLE=1

# Variables & definitions used for silent installation
SILENT_ENABLED=0
SILENT_MMODE=2
SILENT_ACHANGE=1
SILENT_MHOST=0
SILENT_MEXCL=0
SILENT_MPORT=23333
SILENT_SECURITY=0
SILENT_READONLY=0
SILENT_Q0_MODE=0

# General local variables
UPGRADE_SENSOR="n"
VERIFY_MGMT_HOST=0
SECURE_PAM_MGMT_SUPPORTED=0

print()
{
     # If PRINT_TO_CONSOLE is 1, print argument #1 to the console and to the log
     # file with no trailing new line.  Otherwise, only print to the log file.
      if [ "$PRINT_TO_CONSOLE" -eq 1 ]; then
          printf "$1" | tee -a $LOG_FILE
     else
          printf "$1" >> $LOG_FILE
     fi
}

println()
{
     # If PRINT_TO_CONSOLE is 1, print argument #1 to the console and to the log
     # file with a trailing new line.  Otherwise, only print to the log file.
     if [ "$PRINT_TO_CONSOLE" -eq 1 ]; then
          echo "$1" | tee -a $LOG_FILE
     else
          echo "$1" >> $LOG_FILE
     fi
}

run()
{
     # Run the command string specified by argument #1.  If PRINT_TO_CONSOLE is 1, 
     # print stdout and stderr to both the console and the log file.  Otherwise,
     # print only to the log file.
     if [ "$PRINT_TO_CONSOLE" -eq 1 ]; then
          $1 2>&1 | tee -a ${LOG_FILE} 2>&1
     else
          $1 >> $LOG_FILE 2>&1
     fi
}

backup_hbanyware()
{
    if [ -L /usr/sbin/hbanyware ]; then	           # upgrade from OCM 5.1
        rm -rf /usr/sbin/hbanyware
    else
        if [ -d /usr/sbin/hbanyware ]; then        # upgrade from OCM 5.0
            println "Saving /usr/sbin/hbanyware as /usr/sbin/hbanyware.isave"
            rm -rf /usr/sbin/hbanyware.isave
            mv -f /usr/sbin/hbanyware /usr/sbin/hbanyware.isave
        fi
    fi
}

print_help()
{
    echo "Usage: ./install [MODE [OPTION...]]"
    echo "Examples: ./install -q -m=3 -a=1 -h=192.168.1.100 -e=0 -p=30000 -o=0"
    echo "          ./install -q -s=1"
    echo "Install or update the OneCommand Manager Core Kit."
    echo ""
    echo "MODE controls the installation mode: interactive, non-interactive, or silent"
    echo "non-interactive. MODE options are:"
    echo ""
    echo "  -q0, --quiet0              Normal, fully interactive installation (default)."
    echo "                             Same as running the script with no arguments."
    echo "  -q, -q1, --quiet, --quiet1 Non-interactive installation. Users are not"
    echo "                             prompted for input during installation. Users may"
    echo "                             specify install options using the OPTION arguments."
    echo "                             Default settings will be used for all values not"
    echo "                             specified."
    echo "  -q2, --quiet2              Silent, non-interactive installation. Same as non-"
    echo "                             interactive installation, but no output will be"
    echo "                             printed to the console."   
    echo ""
    echo "OPTION arguments are used to specify the OneCommand Manager installation"
    echo "settings in non-interactive and silent non-interactive mode. Default settings"
    echo "will be used for any that are not specified. The available options are:"
    echo ""
    echo "  -s, --sec=<0|1>        Secure Management (0=No, 1=Yes). Default=No."
    echo "                         When this option is selected, all OCM local and remote"
    echo "                         commands require proper OCM group membership on the" 
    echo "                         LDAP domain controller or local machine. This option"
    echo "                         overrides (and therefore is not compatible with)"
    echo "                         ALL options listed below (except -o)."
    echo "  -m, --mmode=<mode>     Management mode. Numbered <mode> options are as follows:"
    echo ""
    echo  "   1   Strictly Local Management : Only manage the adapters on this host."
    echo  "                                   Management of adapters on this host from other"
    echo  "                                   hosts is not allowed."
    echo  "   2   Local Management Plus     : Only manage the adapters on this host."
    echo  "                                   Management of adapters on this host from other"
    echo  "                                   hosts is allowed. (Default)"
    echo  "   3   Full Management           : Manage the adapters on this host and other"
    echo  "                                   hosts that allow it.  Management of the adapters"
    echo  "                                   on this host from another host is allowed."
    echo  "   4   Management Host           : Manage the adapters on this host and other hosts"
    echo  "                                   that allow it.  Management of the adapters on"
    echo  "                                   this host from another host is not allowed."
    echo
 #   echo "                           1 Local Only Management"
 #   echo "                           2 Local Plus Management (default)"
 #   echo "                           3 Full Management"
 #   echo "                           4 Local Plus Management, Read Only"
 #   echo "                           5 Full Management, Read Only"
 #   echo "                           6 Management Host"
    echo "  -r, --readonly=<0|1>   When set to '1', limits client users to passive (monitoring) operations;"
    echo "                         active operations (e.g. download) are prohibited. Default=0."
    echo "  -a, --achange=<0|1>    Allow users to change the management mode after"
    echo "                         installation (0=No, 1=Yes ). Default=1."
    echo "  -h, --mhost=<host>     Register with Management Host <host> (not valid for"
    echo "                         Management Modes 1 and 4). <host> may be the IPv4 or"
    echo "                         IPv6 address or hostname of a remote Management"
    echo "                         Host. '0' (default) disables this functionality."
    echo "  -e, --mexcl=<0|1>      Process requests exclusively from the Management"
    echo "                         Host (0=No, 1=Yes). Default=0."
    echo "  -p, --mport=<portnum>  TCP/IP port number of which management will occur."
    echo "                         Valid port numbers are 1024-65535. '0' will disable"
    echo "                         TCP/IP management to/from remote hosts. Default=23333"
    echo "  -o, --ocv=<0|1>        Install or upgrade the OneCommand Vision Sensor"
    echo "                         (0=No, 1=Yes). Default=0. NOTE: The OCV Sensor is"
    echo "                         only available on certain platforms."
    echo ""
}

add_group_privileges_linux()
{
    # Some files may or may not exist yet. Create them so we can set the privileges.
    touch /${INSTALL_DIR}/rm.log
    touch /${INSTALL_DIR}/cnaboardmgmt.log
    touch /etc/emulexRMPref

    setfacl -m g:ocmadmin:rwx      /${INSTALL_DIR} 
    setfacl -m g:ocmlocaladmin:rwx /${INSTALL_DIR}
    setfacl -m g:ocmuser:rwx       /${INSTALL_DIR}
    setfacl -m g:ocmlocaluser:rwx  /${INSTALL_DIR}

    setfacl -m g:ocmadmin:rx      /${INSTALL_DIR}/hbacmd
    setfacl -m g:ocmlocaladmin:rx /${INSTALL_DIR}/hbacmd
    setfacl -m g:ocmuser:rx       /${INSTALL_DIR}/hbacmd
    setfacl -m g:ocmlocaluser:rx  /${INSTALL_DIR}/hbacmd

    setfacl -m g:ocmadmin:rx      /${INSTALL_DIR}/convert_conf_file 
    setfacl -m g:ocmlocaladmin:rx /${INSTALL_DIR}/convert_conf_file 
    setfacl -m g:ocmuser:rx       /${INSTALL_DIR}/convert_conf_file 
    setfacl -m g:ocmlocaluser:rx  /${INSTALL_DIR}/convert_conf_file 

    setfacl -m g:ocmadmin:rw      /${INSTALL_DIR}/rm.log
    setfacl -m g:ocmlocaladmin:rw /${INSTALL_DIR}/rm.log
    setfacl -m g:ocmuser:rw       /${INSTALL_DIR}/rm.log
    setfacl -m g:ocmlocaluser:rw  /${INSTALL_DIR}/rm.log

    setfacl -m g:ocmadmin:rw      /${INSTALL_DIR}/cnaboardmgmt.log
    setfacl -m g:ocmlocaladmin:rw /${INSTALL_DIR}/cnaboardmgmt.log
    setfacl -m g:ocmuser:rw       /${INSTALL_DIR}/cnaboardmgmt.log
    setfacl -m g:ocmlocaluser:rw  /${INSTALL_DIR}/cnaboardmgmt.log

    setfacl -m g:ocmadmin:rwx      /${INSTALL_DIR}/RMRepository
    setfacl -m g:ocmlocaladmin:rwx /${INSTALL_DIR}/RMRepository
    setfacl -m g:ocmuser:rwx       /${INSTALL_DIR}/RMRepository
    setfacl -m g:ocmlocaluser:rwx  /${INSTALL_DIR}/RMRepository

    # If misc directory doesn't exist yet, create it so we can ensure that
    # the proper privileges are set.
    if [ ! -d /${INSTALL_DIR}/misc ]; then
        mkdir -p /${INSTALL_DIR}/misc
    fi
    setfacl -m g:ocmadmin:rx      /${INSTALL_DIR}/misc > /dev/null 2>&1
    setfacl -m g:ocmlocaladmin:rx /${INSTALL_DIR}/misc > /dev/null 2>&1
    setfacl -m g:ocmuser:rx       /${INSTALL_DIR}/misc > /dev/null 2>&1
    setfacl -m g:ocmlocaluser:rx  /${INSTALL_DIR}/misc > /dev/null 2>&1

    # bugzill 133450
    if [ ! -r /${INSTALL_DIR}/misc/lockFile.sem ]; then
        touch /${INSTALL_DIR}/misc/lockFile.sem
    fi   
    setfacl -m g:ocmadmin:rw      /${INSTALL_DIR}/misc/lockFile.sem > /dev/null 2>&1
    setfacl -m g:ocmlocaladmin:rw /${INSTALL_DIR}/misc/lockFile.sem > /dev/null 2>&1
    setfacl -m g:ocmuser:rw       /${INSTALL_DIR}/misc/lockFile.sem > /dev/null 2>&1
    setfacl -m g:ocmlocaluser:rw  /${INSTALL_DIR}/misc/lockFile.sem > /dev/null 2>&1

    # bugzill 134308
    if [ ! -r /${INSTALL_DIR}/misc/hbahost_lock.sem ]; then
        touch /${INSTALL_DIR}/misc/hbahost_lock.sem
    fi   
    setfacl -m g:ocmadmin:rw      /${INSTALL_DIR}/misc/hbahost_lock.sem > /dev/null 2>&1
    setfacl -m g:ocmlocaladmin:rw /${INSTALL_DIR}/misc/hbahost_lock.sem > /dev/null 2>&1
    setfacl -m g:ocmuser:rw       /${INSTALL_DIR}/misc/hbahost_lock.sem > /dev/null 2>&1
    setfacl -m g:ocmlocaluser:rw  /${INSTALL_DIR}/misc/hbahost_lock.sem > /dev/null 2>&1

    setfacl -m g:ocmadmin:rwx      /${INSTALL_DIR}/Dump
    setfacl -m g:ocmlocaladmin:rwx /${INSTALL_DIR}/Dump
    setfacl -m g:ocmuser:rwx       /${INSTALL_DIR}/Dump
    setfacl -m g:ocmlocaluser:rwx  /${INSTALL_DIR}/Dump

    setfacl -m g:ocmadmin:rw /etc/emulexDiscConfig
    setfacl -m g:ocmadmin:rw /etc/emulexRMConfig
    setfacl -m g:ocmadmin:rw /etc/emulexRMOptions
    setfacl -m g:ocmadmin:rw /etc/emulexRMPref

    setfacl -m g:ocmlocaladmin:rw /etc/emulexDiscConfig
    setfacl -m g:ocmlocaladmin:rw /etc/emulexRMConfig
    setfacl -m g:ocmlocaladmin:rw /etc/emulexRMOptions
    setfacl -m g:ocmlocaladmin:rw /etc/emulexRMPref

    setfacl -m g:ocmuser:rw /etc/emulexDiscConfig
    setfacl -m g:ocmuser:rw /etc/emulexRMConfig
    setfacl -m g:ocmuser:rw /etc/emulexRMOptions
    setfacl -m g:ocmuser:rw /etc/emulexRMPref

    setfacl -m g:ocmlocaluser:rw /etc/emulexDiscConfig
    setfacl -m g:ocmlocaluser:rw /etc/emulexRMConfig
    setfacl -m g:ocmlocaluser:rw /etc/emulexRMOptions
    setfacl -m g:ocmlocaluser:rw /etc/emulexRMPref

    if [ "$KITNAME" = "Enterprise" ];then
        if [ -r /${INSTALL_DIR}/ocmanager ]; then
            setfacl -m g:ocmadmin:rx      /${INSTALL_DIR}/ocmanager 
            setfacl -m g:ocmlocaladmin:rx /${INSTALL_DIR}/ocmanager 
            setfacl -m g:ocmuser:rx       /${INSTALL_DIR}/ocmanager 
            setfacl -m g:ocmlocaluser:rx  /${INSTALL_DIR}/ocmanager 
        fi
        if [ -d /${INSTALL_DIR}/jre ]; then
            setfacl -R -m g:ocmadmin:rx      /${INSTALL_DIR}/jre
            setfacl -R -m g:ocmlocaladmin:rx /${INSTALL_DIR}/jre
            setfacl -R -m g:ocmuser:rx       /${INSTALL_DIR}/jre
            setfacl -R -m g:ocmlocaluser:rx  /${INSTALL_DIR}/jre
        fi
        if [ -d /${INSTALL_DIR}/config ]; then
            setfacl -m g:ocmadmin:rwx       /${INSTALL_DIR}/config
            setfacl -m g:ocmlocaladmin:rwx  /${INSTALL_DIR}/config
            setfacl -m g:ocmuser:rwx        /${INSTALL_DIR}/config
            setfacl -m g:ocmlocaluser:rwx   /${INSTALL_DIR}/config
            setfacl -m g:ocmadmin:rw        /${INSTALL_DIR}/config/*
            setfacl -m g:ocmlocaladmin:rw   /${INSTALL_DIR}/config/*
            setfacl -m g:ocmuser:rw         /${INSTALL_DIR}/config/*
            setfacl -m g:ocmlocaluser:rw    /${INSTALL_DIR}/config/*    
        fi
    fi
}

write_configuration_to_file()
{     
    # write secure management' option into config file
    TF=/etc/emulexRMConfig.tmp
    cat /etc/emulexRMConfig | sed "/SecureManagement/d" > $TF
    if [ "$SILENT_SECURITY" -eq 1 ]; then
       echo "SecureManagement:enabled" >> $TF
    else
       echo "SecureManagement:disabled" >> $TF
    fi
    cp $TF /etc/emulexRMConfig
    rm $TF

    # write server remote operation option to config file      
    TF=/etc/emulexRMConfig.tmp
    cat /etc/emulexRMConfig | sed "/RemoteOperation/d" > $TF
    if [ "$SILENT_MMODE" -eq 1 ]; then
       echo "ServerRemoteOperation:disabled" >> $TF
    else
       echo "ServerRemoteOperation:enabled" >> $TF
    fi
    cp $TF /etc/emulexRMConfig
    rm $TF

    # write client remote operation option to config file  
    TF=/etc/emulexDiscConfig.tmp
    cat /etc/emulexDiscConfig | sed "/RemoteOperation/d" > $TF
    if [ "$SILENT_MMODE" -eq 3 ] || [ "$SILENT_MMODE" -eq 4 ];then
       echo "ClientRemoteOperation:enabled" >> $TF
    else
       echo "ClientRemoteOperation:disabled" >> $TF
    fi    
    cp $TF /etc/emulexDiscConfig
    rm $TF

    # can user make subsequent operation mode changes
    TF=/etc/emulexDiscConfig.tmp
    cat /etc/emulexDiscConfig | sed "/OperationMode/d" > $TF
    if [ "$SILENT_ACHANGE" -eq 1 ]; then
       echo "ClientOperationMode:enabled" >> $TF
    else
       echo "ClientOperationMode:locked" >> $TF
    fi
    cp $TF /etc/emulexDiscConfig
    rm $TF

    TF=/etc/emulexRMOptions.tmp
    cat /etc/emulexRMOptions | sed "/LocalHBAsOnly/d" > $TF
    if [ "$SILENT_MMODE" -eq 3 ] || [ "$SILENT_MMODE" -eq 4 ];then
       echo "LocalHBAsOnly:false" >> $TF
    else
       echo "LocalHBAsOnly:true" >> $TF
    fi
    cp $TF /etc/emulexRMOptions
    rm $TF

    # place user's 'read only' configuration selection into config file
    TF=/etc/emulexDiscConfig.tmp
    cat /etc/emulexDiscConfig | sed "/ClientReadOnlyOperation/d" > $TF
    if [ "$SILENT_READONLY" -eq 1 ]; then
        echo "ClientReadOnlyOperation:enabled" >> $TF
    else
        echo "ClientReadOnlyOperation:disabled" >> $TF
    fi
    cp $TF /etc/emulexDiscConfig
    rm $TF

    # place 'ManagementHost' value into config file
    TF=/etc/emulexRMConfig.tmp
    cat /etc/emulexRMConfig | sed "/ManagementHost/d" > $TF
    if [ "$SILENT_MHOST" != "0" ];then
       echo "ManagementHost:true" >> $TF
    else
       echo "ManagementHost:false" >> $TF
    fi
    cp $TF /etc/emulexRMConfig
    rm $TF

    # place 'tcp/ip socket' port number into config file
    TF=/etc/emulexDiscConfig.tmp
    cat /etc/emulexDiscConfig | sed "/TcpSocketPortNumber/d" > $TF
    echo "TcpSocketPortNumber:$SILENT_MPORT" >> $TF
    cp $TF /etc/emulexDiscConfig
    rm $TF

    # place management host name or ip address value into config file
    TF=/etc/emulexRMConfig.tmp
    cat /etc/emulexRMConfig | sed "/MngmtHostIpAddress/d" > $TF
    echo "MngmtHostIpAddress:$SILENT_MHOST" >> $TF
    cp $TF /etc/emulexRMConfig
    rm $TF

    # place exclusive mgmnt host config value into config file
    TF=/etc/emulexRMConfig.tmp
    cat /etc/emulexRMConfig | sed "/ExclusiveHostMgmnt/d" > $TF
    if [ "$SILENT_MEXCL" -eq 1 ]; then
       echo "ExclusiveHostMgmnt:true" >> $TF
    else
       echo "ExclusiveHostMgmnt:false" >> $TF
    fi
    cp $TF /etc/emulexRMConfig
    rm $TF
}

# create installation log file if necessary
if [ ! -d "/${INSTALL_DIR}" ]; then
   mkdir "/${INSTALL_DIR}"
fi

if [ ! -f ${LOG_FILE} ]; then
   touch ${LOG_FILE}
fi

# Determine OS
if [ -f /etc/redhat-release ]; then
    citrixos=`cat /etc/redhat-release | cut -d' ' -f1`
    oracleos=`grep Oracle /etc/issue | wc -l`
    if [ "$citrixos" = "XenServer" ];then
        citrix_major=`cat /etc/redhat-release | cut -d' ' -f3 | cut -d'.' -f1`
        citrix_minor=`cat /etc/redhat-release | cut -d' ' -f3 | cut -d'.' -f2`
        citrix_rev=`cat /etc/redhat-release | cut -d' ' -f3 | cut -d'.' -f3 | cut -d'-' -f1`
        
        SECURE_PAM_MGMT_SUPPORTED=1
        if [[ $citrix_major -eq 5 ]] && [[ $citrix_minor -eq 6 ]] && [[ $citrix_rev -eq 0 ]]; then
            OS_VER="xenserver-5.6.0"
        elif [[ $citrix_major -eq 5 ]] && [[ $citrix_minor -eq 6 ]] && [[ $citrix_rev -ge 100 ]]; then
            OS_VER="xenserver-5.6-SP2"
        elif [[ $citrix_major -eq 6 ]] && [[ $citrix_minor -eq 0 ]] && [[ $citrix_rev -ge 0 ]]; then
            OS_VER="xenserver-6.0"
        elif [[ $citrix_major -ge 6 ]]; then
            OS_VER="xenserver-6.0"
        else
            println "This is not a supported version of Citrix XenServer."
            println "Aborting installation."
            exit $FAILURE
        fi
    elif [ "$oracleos" = "1" ]; then
        kernel_major_version=$(uname -r | awk -F- '{print$1}' | awk -F. '{print$3}')
        kernel_minor_version=$(uname -r | awk -F- '{print$2}' | awk -F. '{print$1}')

        uekos=`uname -a | grep uek | wc -l`
        SECURE_PAM_MGMT_SUPPORTED=1
        if [ "$uekos" = "1" ]; then
            oracle_major_version=`cat /etc/oracle-release | cut -d' ' -f5 | cut -d. -f1`
            oracle_minor_version=`cat /etc/oracle-release | cut -d' ' -f5 | cut -d. -f2`

            if [ $oracle_major_version -eq 6 ];then
                OS_VER="uek-6"
            elif [ $oracle_major_version -eq 5 ];then
                if [ $oracle_minor_version -lt 6 ]; then
                    println "This is not a supported version of Oracle Enterprise Linux."
                    println "Aborting installation."
                    exit $FAILURE
                else
                    OS_VER="uek-5"
                fi
            else
                println "This is not a supported version of Oracle Enterprise Linux."
                println "Aborting installation."
                exit $FAILURE
            fi
        # only uek installations not detected in above logic is R1 RH5.6 and RH6.0
        # check for these two now
        elif [[ $kernel_major_version -eq 32 ]] && [[ $kernel_minor_version -eq 100 ]]; then 
           oracle_major_version=`cat /etc/redhat-release | cut -d' ' -f7 | cut -d. -f1`
           oracle_minor_version=`cat /etc/redhat-release | cut -d' ' -f7 | cut -d. -f2`

           if [ $oracle_major_version -eq 6 ];then
                OS_VER="uek-6"
           elif [[ $oracle_major_version -eq 5 ]] && [[ $oracle_minor_version -eq 6 ]]; then
                OS_VER="uek-5"
           else
                println "This is not a supported version of Oracle Enterprise Linux."
                println "Aborting installation."
                exit $FAILURE
           fi
        else      
           oracle_major_version=`cat /etc/redhat-release | cut -d' ' -f7 | cut -d. -f1`
           oracle_minor_version=`cat /etc/redhat-release | cut -d' ' -f7 | cut -d. -f2`

           # else these are oracle OEL installations and not UEK; therefore install rhel binaries
           if [ $oracle_major_version -eq 6 ];then
                OS_VER="rhel-6"
           elif [[ $oracle_major_version -eq 5 ]] && [[ $oracle_minor_version -ge 6 ]]; then
                OS_VER="rhel-5"
           else
                println "This is not a supported version of Oracle Enterprise Linux."
                println "Aborting installation."
                exit $FAILURE
           fi
        fi
    elif [ "$citrixos" = "CentOS" ];then
        CENTOS_RT="-f3"
        centos=`cat /etc/redhat-release | grep 6.0 -q --quiet`
        if [ $? -eq 0 ]; then
           CENTOS_RT="-f4"
        fi

        SECURE_PAM_MGMT_SUPPORTED=1
        rhel_major_version=`cat /etc/redhat-release | cut -d' ' ${CENTOS_RT} | cut -d. -f1`
        rhel_minor_version=`cat /etc/redhat-release | cut -d' ' ${CENTOS_RT} | cut -d. -f2`

        if [ "$rhel_major_version" -eq 5 ];then
           if [ "$rhel_minor_version" -lt 5 ];then
               println "This is not a supported version of CentOS operating system."
               println "Aborting installation."
               exit $FAILURE
           else
               OS_VER="rhel-5"
           fi
        elif [ "$rhel_major_version" -eq 6 ];then
           OS_VER="rhel-6"
        else
           println "This is not a supported version of CentOS operating system."
           println "Aborting installation."
           exit $FAILURE
        fi
    else
        rhel_major_version=`cat /etc/redhat-release | cut -d' ' -f7 | cut -d. -f1`
        rhel_minor_version=`cat /etc/redhat-release | cut -d' ' -f7 | cut -d. -f2`
        if [ "$rhel_major_version" -eq 5 ];then
            if [ "$rhel_minor_version" -lt 4 ];then
                println "This is not a supported version of Red Hat 5"
                println "Aborting installation."
                exit $FAILURE
            else
                SECURE_PAM_MGMT_SUPPORTED=1
                OS_VER="rhel-5"
            fi
        elif [ "$rhel_major_version" -eq 6 ];then
            OS_VER="rhel-6"
            SECURE_PAM_MGMT_SUPPORTED=1
        else
            println "This is not a supported version of Red Hat."
            println "Aborting installation."
            exit $FAILURE
        fi
    fi
elif [ -f /etc/SuSE-release ]; then
    sles_major_version=`cat /etc/SuSE-release | grep VERSION | awk '{ print $3 }'`
    sles_minor_version=`cat /etc/SuSE-release | grep PATCHLEVEL | awk '{ print $3 }'`
    if [ "$sles_major_version" -eq 10 ];then
        if [ "$sles_minor_version" -lt 3 ];then
            println "This is not a supported version of SuSE Linux Enterprise Server 10."
            println "Aborting installation."
            exit $FAILURE
        else
            SECURE_PAM_MGMT_SUPPORTED=1
            OS_VER="sles-10"
        fi
    elif [ "$sles_major_version" -eq 11 ];then
        if [ "$sles_minor_version" -lt 1 ];then
            println "This is not a supported version of SuSE Linux Enterprise Server 11."
            println "Aborting installation."
            exit $FAILURE
        else
            OS_VER="sles-11"
            SECURE_PAM_MGMT_SUPPORTED=1
        fi
    else
        println "This is not a supported version of SuSE Linux Enterprise Server."
        println "Aborting installation."
        exit $FAILURE
    fi
else
    println "ERROR: Cannot determine Linux distribution."
    println "Aborting installation."
    exit $FAILURE
fi

# Interate through all command line arguments, handling them appropriately.
for arg in $*
do
     case "$arg" in
          "-q0" | "--quiet0" )
          # Normal interactive mode
          SILENT_ENABLED=0
          SILENT_Q0_MODE=1
          ;;
           
          "-q" | "-q1" | "--quiet" | "--quiet1" )
          # Silent mode enabled
          SILENT_ENABLED=1
          ;;
    
          "-q2" | "--quiet2" )
          # Extra silent mode enabled.  No ouput sent to console.
          SILENT_ENABLED=1
          PRINT_TO_CONSOLE=0
          ;;
          
           "-s="* | "--sec="* )
          # Secure Management specified
          SILENT_SECURITY=`echo $arg | cut -d= -f 2`
          if [ "$SILENT_SECURITY" -gt 1 ]; then
               println "ERROR: Invalid secure management entry: ${SILENT_SECURITY}"
               print_help
               exit 1;
          fi
          if [ $SECURE_PAM_MGMT_SUPPORTED -ne 1 ]; then         
               println "ERROR: OCM Secure Management feature is not supported on this distribution."
               exit 1;
          fi
          ;;

          "-m="* | "--mmode="* )
          # Management Mode specified
          SILENT_MMODE=`echo $arg | cut -d= -f 2`
          if [ "$SILENT_MMODE" -lt 1 ] || [ "$SILENT_MMODE" -gt 4 ]; then
               println "ERROR: Invalid mmode entry: ${SILENT_MMODE}"
               print_help
               exit 1;
          fi
          ;;
          
          "-a="* | "--achange="* )
          # Allow Management Mode to be changed after installation
          SILENT_ACHANGE=`echo $arg | cut -d= -f 2`
          if [ "$SILENT_ACHANGE" != "0" ] && [ "$SILENT_ACHANGE" != "1" ]; then
               println "ERROR: Invalid achange entry: ${SILENT_ACHANGE}"
               print_help
               exit 1;
          fi
          ;;
        
          "-h="* |"--mhost="* )
          # Management Host specified.
          SILENT_MHOST=`echo $arg | cut -d= -f 2`
          
          # If not "0", verify mhost entry later after the architecture-specific kits have been unpacked.
          if [ "$SILENT_MHOST" != "0" ];then
               VERIFY_MGMT_HOST=1        
          fi      
          ;;
        
          "-e="* | "--mexcl="* )
          # Specifies whether non-management hosts should process requests 
          # exclusively from the Management Host specified by mhost.
          SILENT_MEXCL=`echo $arg | cut -d= -f 2`
          if [ "$SILENT_MEXCL" != "0" ] && [ "$SILENT_MEXCL" != "1" ]; then
               println "ERROR: Invalid mexcl entry: ${SILENT_MEXCL}"
               print_help
               exit 1;
          fi        
          ;;
        
          "-r="* | "--readonly="* )
          # Specifies whether ocm clients are restricted to passive (read-only)
          # operations.
          SILENT_READONLY=`echo $arg | cut -d= -f 2`
          if [ "$SILENT_READONLY" != "0" ] && [ "$SILENT_READONLY" != "1" ]; then
               println "ERROR: Invalid readonly entry: ${SILENT_READONLY}"
               print_help
               exit 1;
          fi        
          ;;
        
          "-p="* | "--mport="* )
          # Set the port number by which TCP/IP management will occur, or 
          # disable TCP/IP management (mport=0).
          SILENT_MPORT=`echo $arg | cut -d= -f 2`
          
          # Validate the TCP/IP port number
          test_value=`echo $SILENT_MPORT | tr -d "0123456789"`
          if [ "$test_value" != "" ] || [ "$SILENT_MPORT" -lt 1024 ] || [ "$SILENT_MPORT" -gt 65535 ];then
               if [ "$SILENT_MPORT" != "0" ]; then
                    println "ERROR: Invalid mport entry: ${SILENT_MPORT}"
                    print_help
                    exit 1
               fi
          fi
          ;;
        
          "-o="* | "--ocv="* )
          # Specifies whether the OneCommand Vision Sensor should be installed.
          UPGRADE_SENSOR=`echo $arg | cut -d= -f 2`
          if [ "$UPGRADE_SENSOR" = "0" ]; then
               UPGRADE_SENSOR="n"
          elif [ "$UPGRADE_SENSOR" = "1" ]; then
               UPGRADE_SENSOR="y"
          else
               println "ERROR: Invalid ocv entry: ${UPGRADE_SENSOR}"
               print_help
               exit 1
          fi
          ;;
   
          "-?" | "--help" )
          # Print the help message
          print_help
          exit 0
          ;;
               
          * )
          # Default: Invalid parameter
          println "ERROR: Invalid argument: ${arg}"
          print_help
          exit 1;
          ;;
     esac
done

# Return an error if the user specified additional install parameters, but didn't 
# specify silent install (-q, -q1, -q2, --quiet, --quiet1, --quiet2). The exceptions
# are the -q0 and --quiet0 arguments.
if [ "$SILENT_ENABLED" -eq 0 ] && [ "$#" -gt 0 ]; then
     if [ "$#" -eq 1 ] && [ "$SILENT_Q0_MODE" -eq 1 ]; then
          # Do nothing.
          echo ""
     else
          println "ERROR: Installation options are only accepted for non-interactive installs."
          println "       To perform a non-interactive install, add the -q, -q1, -q2, --quiet,"
          println "       --quiet1, or --quiet2 option."  
          print_help
          exit 1;
     fi
fi

# Check for conflicting settings (silent mode only).
if [ "$SILENT_ENABLED" -eq 1 ]; then
    
     # Return an error if the user sets both 'secure management' mode and any other mode 
     # of operation       
     if [ "$SILENT_SECURITY" -eq 1 ]; then  
          if [ "$SILENT_MMODE" -ne 2 ] || [ "$SILENT_ACHANGE" -ne 1 ] || [ "$SILENT_MHOST" -ne 0 ] || [ "$SILENT_MEXCL" -ne 0 ] || [ "$SILENT_READONLY" -ne 0 ] || [ "$SILENT_MPORT" -ne 23333 ]; then
               println "ERROR: When enabling 'secure management' mode, all other options "
               println "       (except -o) are overridden (and therefore disallowed). "
               exit 1  
          fi
     fi
     
     # Return an error if the user both sets a remote Management Host and sets the management 
     # mode to "Local Only" or "Management Host"       
     if [ "$SILENT_MHOST" != "0" ]; then  
          if [ "$SILENT_MMODE" -eq 1 ] || [ "$SILENT_MMODE" -eq 4 ]; then
               println "ERROR: A remote Management Host (--mhost=?) cannot be specified on a system"
               println "       configured as a Management Host (--mmode=4) or running in Local"
               println "       Only mode (--mmode=1)"
               exit 1  
          fi
     fi
     
     # Return an error if the user has specified exclusive management by the Management Host, yet
     # has not entered a Management Host, or is running as a Management Host, or is running in Local Only mode..
     if [ "$SILENT_MEXCL" -eq 1 ]; then
          if [ "$SILENT_MMODE" -eq 4 ]; then
               println "ERROR: Exclusive mangement by a Management Host (--mexcl=1) is not allowed on"
               println "       a Management Host (--mmode=6)."
               exit 1
          fi

          if [ "$SILENT_MMODE" -eq 1 ]; then
               println "ERROR: Exclusive mangement by a Management Host (--mexcl=1) cannot be specified"
               println "       in conjunction with Local Only management (--mmode=1)."
               exit 1
          fi
                              
          if [ "$SILENT_MHOST" = "0" ]; then
               println "ERROR: In order to set exclusive managment by a Management Host (--mexcl=1),"
               println "       a Management Host address must be entered (--mhost=?)."
               exit 1
          fi
     fi
     
     # Return an error if the user configures the system as a Management Host, but also
     # requests client be read only.
     if [ "$SILENT_READONLY" -eq 1 ] && [ "$SILENT_MMODE" -eq 4 ]; then
          println "ERROR: read only option (--readonly=1) not valid on a Management"
          println "       Host (--mmode=4)."
          exit 1
     fi

     # Return an error if the user specifies both Local Only management mode and sets a TCP/IP port number
     if [ "$SILENT_MPORT" -ne 0 ] && [ "$SILENT_MPORT" -ne 23333 ] && [ "$SILENT_MMODE" -eq 1 ]; then
          println "ERROR: A TCP/IP port number cannot be specified (--mport=?) in conjunction"
          println "       with Local Only management (--mmode=1)."
          exit 1
     fi
     
fi


cd $DIR
if [ ! -d "${Arch}/${OS_VER}" ]; then
    println "This operating system is not supported by this $KITNAME Kit."
    println "Aborting installation."
    exit $FAILURE
else
    cd ${Arch}/${OS_VER}
fi


RPMHBAAPI=`ls elxocmlibhbaapi-*.rpm`
RPMCORE=`ls elxocmcore-*.rpm`
if [ -f elxocmgui-*.rpm ] && [ -f elxocmjvm-*.rpm ]; then
    RPMGUI=`ls elxocmgui-*.rpm`
    RPMJVM=`ls elxocmjvm-*.rpm`
    KITNAME="Enterprise"
else
    RPMGUI=""
    RPMJVM=""
    KITNAME="Core"
fi

if [ -f elx-udev-config*.rpm ]; then
    RPMUDEV=`ls elx-udev-config*.rpm`
else
    RPMUDEV=""
fi

# stop OCV daemons if necessary
if [ "$KITNAME" = "Enterprise" ];then
   if [ "$OS" = "Linux" ];then
      SENSOR_PRESENT=`rpm -qa | grep OneCommandVisionSensor`
      if [ $? -eq 0 ]; then
          cd ../../
          ./OneCommandVisionSensor*/stop-sensor
          cd ${Arch}/${OS_VER}
      fi
   fi
fi
  
# disallow corekit installation overtop enterprise kit (bug 107303 and 107538) 
if [ "$KITNAME" = "Core" ];then
    if [ "$OS" = "Linux" ];then
        rpm -q --quiet elxocmgui
        if [ $? -eq 0 ]; then
           println "Error: Attempting to install OneCommand Manager Core Kit over more fully featured OneCommand Manager Enterprise Kit."
           println "If this is intentional, please uninstall Enterprise Kit before attempting Core Kit installation."
           exit -1
       fi
    fi
fi

println "Beginning OneCommand Manager $KITNAME Kit Installation..."


# Decide if libHBAAPI install is neccessary
HBAAPI_LIST=`ls elxocmlibhbaapi-*`
for hbaapi_rpm in $HBAAPI_LIST
do
    rpm -U --test --replacepkgs $hbaapi_rpm > /dev/null 2>&1
    if [ $? -eq 0 ]; then
        
        RPMLIST="${hbaapi_rpm} ${RPMLIST}"
    fi
done

# Install OCM
RPMLIST="$RPMLIST $RPMCORE $RPMJVM $RPMGUI $RPMUDEV"
for RPM in $RPMLIST
do
    println "Installing ./${Arch}/${OS_VER}/$RPM"
done
println ""
rpm -U --replacepkgs $RPMLIST
STATUS=$?
if [ "$STATUS" != 0 ]; then
    println "OneCommand Manager $KITNAME Kit install failed."
    exit $STATUS
fi

#  is this a silent install
if [ "$SILENT_ENABLED" -eq 1 ]; then

   # Verify the --mhost argument if entered.
   if [ "$VERIFY_MGMT_HOST" = "1" ]; then
       if [ ! -f /${INSTALL_DIR}/elxchkaddr ]; then
           println "WARNING: Could not find elxchkaddr. Management Host address cannot"
           println "         be validated. Using anyway."
       fi
      
       # Call TCP/IP address verification script
       #           -1 = no address argument
       #            0 = good address
       #            1 = bad address
       #            2 = local host
       #            3 = host name not found
       /${INSTALL_DIR}/elxchkaddr $SILENT_MHOST > /dev/null 2>&1
       return_status=$?
          
       println ""
       println ""
          
       if [ $return_status -eq 0 ]; then
            println "Address $SILENT_MHOST successfully validated. Setting as Management Host."
       elif [ $return_status -eq 1 ]; then
            println "ERROR: Host address $SILENT_MHOST appears to be an invalid IP address or hostname."
            exit 1
       elif [ $return_status -eq 2 ]; then
            println "ERROR: Host address $SILENT_MHOST appears to be the local host address."
            println "Address should be the address of a remote machine."
            println "Please retry using a remote address or hostname."
            exit 1
       elif [ $return_status -eq 3 ]; then
            println ""
            println "WARNING: Management Host $SILENT_MHOST not found.  Using anyway."
            println ""
       else
            println "ERROR: Host address $SILENT_MHOST appears to be an invalid IP address or hostname."
            exit 1
       fi   
   fi

   #write /etc files
   write_configuration_to_file

   # start the daemons
   /${INSTALL_DIR}/start_ocmanager > /dev/null 2>&1
else
   cd ${DIR}

   # obtain user selections for remote management operating mode 
   if [ $SECURE_PAM_MGMT_SUPPORTED -ne 1 ]; then         
      /${INSTALL_DIR}/set_operating_mode 45
   else
      /${INSTALL_DIR}/set_operating_mode 25
   fi

   if [ $? -eq 1 ]; then
      exit 1
   fi
   # obtain user selections for OCV product installation 
   if [ "$KITNAME" = "Enterprise" ];then
      if [ "$Arch" != "ppc64" ] && [ "$Arch" != "ia64" ];then
          if [ "$OS" = "Linux" ];then
             SENSOR_PRESENT=`rpm -qa | grep OneCommandVisionSensor`
          else
             SENSOR_PRESENT=`pkginfo | grep OCVsensor`
          fi
          if [ $? -eq 0 ]; then
             stay_in_loop=1
             while [ $stay_in_loop -eq 1 ]
             do
                echo "OneCommandVision Sensor detected"
                echo "Do you want to upgrade the sensor kits?"
                echo "Enter y to upgrade"
                echo "Enter n not to upgrade"
                echo -n "Enter the letter 'y' or 'n': "
                read upgrade_sensor
                if [ "$upgrade_sensor" = "Y" ] || [ "$upgrade_sensor" = "y" ]; then
                        stay_in_loop=0;
                        echo "Sensor kits will be upgraded"
                elif [ "$upgrade_sensor" = "N" ] || [ "$upgrade_sensor" = "n" ]; then
                        echo "Sensor kits will not be upgraded."
                        stay_in_loop=0;
                        ./OneCommandVisionSensor*/start-sensor
                else
                        echo "Invalid Selection "
                fi
             done
          else
             stay_in_loop=1
             while [ $stay_in_loop -eq 1 ]
             do
                echo "Do you want to install the sensor kits?"
                echo "Enter y to install"
                echo "Enter n not to install"
                echo -n "Enter the letter 'y' or 'n': "
                read upgrade_sensor
                if [ "$upgrade_sensor" = "Y" ] || [ "$upgrade_sensor" = "y" ]; then
                       echo "Sensor kits will be installed"
                       stay_in_loop=0;
                elif [ "$upgrade_sensor" = "N" ] || [ "$upgrade_sensor" = "n" ]; then
                       echo "Sensor kits will not be installed."
                       stay_in_loop=0;
                else
                       echo "Invalid Selection "
                fi
             done
          fi

          if [ "$upgrade_sensor" = "y" ]; then
              echo "Proceeding to sensor installation"
              cd `pwd`;
              ./OneCommandVisionSensor*/install-sensor
              install_sensor_status=$?;
              if [ $install_sensor_status != 0 ] && [ $install_sensor_status != 1 ]; then
                 ocv_install_error=1;
              fi
              cp -r ./OneCommandVisionSensor* /${INSTALL_DIR}/scripts
          else
              # fix for Bugzilla 52950 to prevent erroneous directory creation
              OCV_DIRECTORY_PRESENT=`ls /${INSTALL_DIR}/scripts | grep OneCommandVis`
              if [ "$OCV_DIRECTORY_PRESENT" = "" ]; then
                 mkdir /${INSTALL_DIR}/scripts/OneCommandVisionSensor
                 cp ./OneCommandVisionSensor*/uninstall-sensor /${INSTALL_DIR}/scripts/OneCommandVisionSensor
              fi
          fi
      fi
   fi
fi

#if secure management is enabled, set file permissions
SECURE_OPERATION_ENABLED=`cat /etc/emulexRMConfig | grep SecureManagement:enabled`
if [ "$SECURE_OPERATION_ENABLED" != "" ]; then
   # secure management file permissions changes
   add_group_privileges_linux > /dev/null 2>&1
fi

# remove script that changes operating mode if installer selects this option
no_mode_change=`cat /etc/emulexDiscConfig | grep ClientOperationMode | grep locked`
if [ -n "$no_mode_change" ];then
    rm -f /${INSTALL_DIR}/set_operating_mode
fi

backup_hbanyware

if [ $ocv_install_error -eq 0 ]; then
    println "OneCommand Manager $KITNAME Kit install completed successfully."
else
    println "Warning: The OC Vision Sensor Module failed to install."
    println "This does NOT affect the OneCommand Manager $KITNAME Kit install,"
    println "which has completed successfully."
fi

println ""
if [ "$KITNAME" = "Enterprise" ];then
    println "Type:    /${INSTALL_DIR}/ocmanager"
    println "to run the OneCommand Manager GUI client."
else
    println "Type:    /${INSTALL_DIR}/hbacmd"
    println "to run the OneCommand Manager CLI utility."
fi
println "" 

exit $SUCCESS
  
