#! /bin/bash
#################################################################################
#SCRIPT:perf_tune_benet.sh                                                      #
#AUTHOR: Madhusudana Reddy                                                      # 
#DATE:23-APR-2013                                               	        #
#Version:2.5                                                                    # 
#PLATFORM:Linux                                                                 #
#PURPOSE:                                                                       #
##   TO get best NIC perf or To Optimise Latency or To acehive best PPS         #  
##   1. Optimal pinning of IRQs in Nehalem and Romeley processors.              #
##   2. Tunning kernel parameters like TCP & UDP rmem & wmem 			#
##+M 3. Changed RFC2544(PPS) argument option to -p instead of -r                #
##+A 4. Added option [ -r ] to restore kernel parameters                        #
##+A 5. Added numa-aware support                                                #
##+A 6. Added remove "intel_iommu" print                                        #
##+A 7. Restart irqtables, with -r option					#
##+A 8. Checking whether OS support NUMA or not(v2.5)                           #
# Usage : perf_tune_benet.sh [-l]/[-t]/[-p]/[-r] eth<X>  eth<Y> ...		#
#################################################################################
htenable()
{
	index_phy=`expr $pcores - 1`
	i=0; k=0
	while [[ $i -le $index_phy ]] ; do
		list=$(grep ',' /sys/devices/system/cpu/cpu$i/topology/thread_siblings_list)
		j=0
		while [[ $j -le $index_cores ]] ; do
			id=$(grep ',' /sys/devices/system/cpu/cpu$j/topology/thread_siblings_list)
			if [[ "$list" == "$id" ]] ; then
				array[$k]=$((2**j)) ;((k++));fi
				((j++))
		done
		((i++))
	done
		
}
setirq()
{
        if [[ "$ARG" = "P" ]] ;then
		ifconfig $1 txqueuelen 30000
	else
		ifconfig $1 txqueuelen 10000
	fi
	IRQS=$(cat /proc/interrupts | grep -w $1 | awk '{print $1}' | sed 's/://')
	if [[ -z "$IRQS" ]] ; then
		echo "No IRQs found for $1"
	else
       		echo Discovered irqs for $1: $IRQS
		if [[ $2 -eq 0 ]] ; then 
			mask=0 
		else
			mask=$index
		fi
		for IRQ in $IRQS 
		do 
			echo Assign irq $IRQ mask 0x$(printf "%x" ${array[$mask]})
			echo $(printf "%x" ${array[$mask]}) > /proc/irq/$IRQ/smp_affinity
			((mask++))
			if [[ $mask -gt $index_cpus ]] ; then 
				mask=$((RANDOM%$index_cpus+0))
			fi 
		done
			index=$mask
	fi
	echo
}
numa_node()
{
	node_num=$2
	j=0 ; k=0
        while [[ $j -le $index_cores ]] ; do
        	id=$(cat /sys/devices/system/cpu/cpu$j/topology/physical_package_id) ##Checking physical CPU node for the cpu core
        	if [[ "$node_num" == "$id" ]] ; then
        		array[$k]=$((2**j)) ;((k++));fi
        ((j++))
  	 done
	setirq $1
}
bios_opt()
{
	cat /proc/cmdline |grep intel_iommu &> /dev/null
	if [[ $? -eq 0 ]] ; then
		echo "intel_iommu is enabled on your OS, Please remove it from your grub.conf/menu.lst file, to get better throughput data"
	fi
	echo "Please enable below options in SYSTEM BIOS, if available."
	echo "1.System settings->Operating Modes = Maximum Performance OR Power management->Performance = Performance "
	echo "2.Power->Workload Configuration = I/O Sensitive"
}
multisock()
{
i=0
while [[ $i -lt $socket ]]
do
	j=0 ; k=0
        while [[ $j -le $index_cores ]] ; do
        	id=$(cat /sys/devices/system/cpu/cpu$j/topology/physical_package_id) ##Checking physical CPU node for the cpu core
        	if [[ "$i" == "$id" ]] ; then
        		array[$k]=$((2**j)) ;((k++));fi
        ((j++))
  	 done
	if [[ -z "$1" ]] ; then
		bios_opt
		exit	
	fi
	setirq $1
	shift
	((i++))
	if [[ $i -ge $socket ]] && [[ $i -le $args ]] ; then
		i=0
	fi

done
}
tuning()
{
	echo 64000000 > /proc/sys/net/core/rmem_default
        echo 64000000 > /proc/sys/net/core/rmem_max
        echo 32000000 > /proc/sys/net/core/wmem_default
        echo 32000000 > /proc/sys/net/core/wmem_max
}
rfc2544()
{
	ethtool -C $1 adaptive-rx off  &> /dev/null
	ethtool -C $1 adaptive-tx off  &> /dev/null
	ethtool -C $1 rx-usecs 128  &> /dev/null
	ethtool -A $1 autoneg off rx off tx off  &> /dev/null
}
throughput()
{	
	echo 4096 87380 16777216 > /proc/sys/net/ipv4/tcp_rmem
        echo 4096 65536 16777216 > /proc/sys/net/ipv4/tcp_wmem
	echo 500000 > /proc/sys/net/core/netdev_max_backlog
        echo 0 > /proc/sys/net/ipv4/tcp_timestamps
        echo 0 > /proc/sys/net/ipv4/tcp_sack 
        echo 0 > /proc/sys/net/ipv4/tcp_ecn 

}
latency_check()
{
	echo 1 > /proc/sys/net/ipv4/tcp_low_latency
	ethtool -C $1 adaptive-rx off rx-usecs 0 &> /dev/null
}
default_sysparams()
{
	sysctl -a | grep net| grep wmem | grep -v udp > /tmp/sysparams
	sysctl -a | grep net| grep rmem | grep -v udp >> /tmp/sysparams
	sysctl -a | grep max_backlog >> /tmp/sysparams
	sysctl -a | grep timestamp >> /tmp/sysparams
	sysctl -a | grep tcp_sack >> /tmp/sysparams
	sysctl -a | grep tcp_ecn >> /tmp/sysparams
	sysctl -a | grep low_latency >> /tmp/sysparams
}


############# MAIN  Starts Here ##############################

if [[ -f /etc/SuSE-release ]] ; then
        SuSEfirewall2 stop &> /dev/null
else
        service iptables stop &> /dev/null
        service ip6tables stop &> /dev/null
	echo 0 > /selinux/enforce &> /dev/null
fi

socket=`cat /proc/cpuinfo | grep "physical id"|sort -n |uniq|wc -l`
total_cores=`cat /proc/cpuinfo | grep processor | wc -l`
tcores=`egrep 'siblings|cpu cores' /proc/cpuinfo | head -1|  awk -F":" '{print $2}' |bc `
pcores=`egrep 'siblings|cpu cores' /proc/cpuinfo | head -2|tail -1|  awk -F":" '{print $2}'|bc`
index_cores=`expr $total_cores - 1`                     
HT_ENABLED=0
if [[ "$1" != "-r" ]] ; then
	default_sysparams
fi
case $1 in
	-l)	shift
		echo -e "\033[1mOptimizing for Latency :\033[m"
		for intf in $*
		do
			latency_check $intf
		done
		tuning
	    	;;
	-t) 	shift
		echo -e "\033[1mOptimizing for Throughput :\033[0m"
		throughput
		tuning
	   	;;
	-p) 	ARG=P
		shift
		echo -e "\033[1mOptimizing for RFC2544[PPS] :\033[0m"
		for intf in $*
		do
			rfc2544 $intf
		done
		tuning
	    	;;
	-r) 	shift
		echo -e "\033[1mRestoring default kernel parms :\033[0m"
		if [[ ! -f /tmp/sysparams ]] ; then
			echo "System already running with default kernel parms"
		else	
			sysctl -p /tmp/sysparams  &> /dev/null
			rm -rf /tmp/sysparams &> /dev/null
		fi
		for intf in $*
		do
			ifconfig $1 txqueuelen 1000  &> /dev/null 
	                ethtool -C $1 adaptive-rx on  &> /dev/null
	                ethtool -C $1 adaptive-tx on  &> /dev/null  
			ethtool -A $1 autoneg off rx off tx off  &> /dev/null
			service irqbalance restart &> /dev/null
		done
		exit
		;;
	*) 	echo "Use -l for latency, -p for PPS/rfc2544, -t for throughput and -r to restore sysparams as first argument
		and eth[x] eth[y] assecond argument and so on.."
		exit
esac
# check for irqbalance running
IRQBALANCE_ON=`ps ax | grep -v grep | grep -q irqbalance; echo $?`
if [[ "$IRQBALANCE_ON" == "0" ]] ; then
	service irqbalance stop &> /dev/null
       	echo "Interrupt for specified interfaces are pinned. irqbalance service has been stopped" 
fi
#Check for Hyper Therad Enabled
if [[ $tcores -ne $pcores ]] ; then
	echo "SMT/HT is enabled now.  Disabling SMT/HT may give better performance in some work loads." 
	index_cpus=`expr $tcores - 1`
	HT_ENABLED=1
else
	index_cpus=`expr $pcores - 1`
fi
#Check for number of CPU populated sockets
if [[ $socket -gt 1 ]] ; then
	if [[ -f /sys/class/net/$1/device/numa_node ]] ; then
		node=$(cat /sys/class/net/$1/device/numa_node)
		if [[ $node -eq -1 ]] ; then
			args=$#
			multisock $*    ##multisocket cpu 
		else
			for intf in $*
			do
				node=$(cat /sys/class/net/$intf/device/numa_node)
				numa_node $intf $node
			done
		fi
	else
		args=$#
		multisock $*    ##multisocket cpu 
	fi	
else
	if [[ $HT_ENABLED -eq 1 ]] && [[ $# -gt 1 ]] ; then
		htenable
	else
		k=0
		for j in `seq 0 1 $index_cores`
		do
			array[k]=$((2**j))
			((k++))
		done
	fi
	l=0	
	for intrf in $*
	do
		setirq $intrf $l
		if [[ $total_cores -gt 8 ]] ; then
			((l++))
		fi	
	done
fi
bios_opt
