#!/bin/bash
#
###########################################################
#
# (C) Copyright IBM Corporation 2007.
#       Released under GPL v2.
#       Author : Donjuma Lee (donslee@us.ibm.com)
#
# VERSION: 2.02-elx
#
# contributions from Joel Diaz (joeldiaz@us.ibm.com)
# modified by Brian Bothwell (brian.bothwell@emulex.com)
#
###########################################################

# driver tool path
DRVTL_PATH="/usr/local/bin"
# ensure a working PATH
export PATH=/bin:/usr/bin:${DRVTL_PATH}:${PATH}

# Emulex installation scripts write extra logging to the
# IBM support directory. Make sure that directory exists.
ELXLOG="/var/log/IBM_Support/elx_driver_install.log"
if [ ! -d /var/log/IBM_Support ]; then
    mkdir -p /var/log/IBM_Support
fi

# count errors
ERROR_COUNT=0

# locatin of the installer rpm
TOOL_DIR="tools"

# set the application directory
APPS_DIR="apps"

INSTALLER_PATH="$(dirname $0)"
cd $INSTALLER_PATH 2> /dev/null

# installer setup
IBM_DRVTL="${TOOL_DIR}/ibm-driver-tool.pl"
INSTALL_OPTIONS="--yes $@"

# basic package health checks
if [ ! -f ${IBM_DRVTL} ]; then
    echo -e "Installation script not found: ${IBM_DRVTL}"
    echo -e "Installation script not found: ${IBM_DRVTL}" >> ${ELXLOG}
    echo -e "Exiting..."
    exit 1
fi

# run the installer with default and user provided options
echo -e "\n\n INSTALL_OPTIONS ${INSTALL_OPTIONS}\n"
echo -e "\n\n INSTALL_OPTIONS ${INSTALL_OPTIONS}\n" >> ${ELXLOG}
${IBM_DRVTL} ${INSTALL_OPTIONS}
EXIT_CODE=$?

# if the package was successfully installed
# install any apps
if [ -d ${APPS_DIR} ] && [ ${EXIT_CODE} -eq 0 ]; then
    ELXLOG="/var/log/IBM_Support/elx_apps_install.log"
    echo -e "Applications directory \"${APPS_DIR}\" found, installing included RPMs"
    echo -e "Applications directory \"${APPS_DIR}\" found, installing included RPMs" >> ${ELXLOG}
    ${APPS_DIR}/elx_install.sh -q >> ${ELXLOG}
    RETURN_CODE=$?
    
    if [ $RETURN_CODE -ne 0 ]; then
        if [[ $(rpm -qa | grep elxocm) ]]; then
            echo -e "OneCommand is already installed"
            echo -e "OneCommand is already installed" >> ${ELXLOG}
        else
            echo -e "An unknown error occurred while installing OneCommand"
            echo -e "An unknown error occurred while installing OneCommand" >> ${ELXLOG}
            EXIT_CODE=${RETURN_CODE}
        fi
    else
        # The OCM installation script starts the OCM services. They must
        # be stopped or UXSPi will hang waiting for all processes to end.
        /usr/sbin/ocmanager/stop_ocmanager >> ${ELXLOG}
    fi
fi

# exit with the installers exit code
exit ${EXIT_CODE}